/*
 * MORPHO GENERATION-F, Open Source Groupware Solutions
 * http://www.mintjams.jp/
 *
 * Copyright (c) 2004, MintJams, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *	  this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *	  this list of conditions and the following disclaimer in the documentation
 *	  and/or other materials provided with the distribution.
 * 3. Neither the name of the MintJams, Inc. nor the names of its contributors
 *	  may be used to endorse or promote products derived from this software
 *	  without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package jp.co.mintjams.generation.base.util;

import java.io.ByteArrayInputStream;
import java.io.Serializable;
import java.io.UnsupportedEncodingException;
import java.security.Key;
import java.security.cert.CertificateEncodingException;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;

import javax.crypto.SecretKey;
import javax.crypto.spec.SecretKeySpec;

import jp.morpho.util.Base64Decoder;
import jp.morpho.util.Base64Encoder;

/**
 * 証明書や鍵を操作するユーティリティです。
 */
public class KeyTools implements Serializable
{

	/**
	 * KeyTools を作成します。
	 */
	public KeyTools()
	{
		super();
	}

	/**
	 * 証明書を Base64 で符号化します。
	 *
	 * @param certificate 証明書
	 * @return Base64 で符号化された証明書
	 */
	public synchronized String encodeCertificate(X509Certificate certificate) throws CertificateEncodingException, UnsupportedEncodingException
	{
		Base64Encoder encoder = new Base64Encoder();
		return new String(encoder.encode(certificate.getEncoded()), "ISO-8859-1");
	}

	/**
	 * Base64 で符号化された証明書を復号化します。
	 *
	 * @param base64Certificate Base64 で符号化された証明書
	 * @return 証明書
	 */
	public synchronized X509Certificate decodeCertificate(String base64Certificate) throws CertificateException
	{
		Base64Decoder decoder = new Base64Decoder();
		byte[] bytes = decoder.decode(base64Certificate);
		CertificateFactory factory = CertificateFactory.getInstance("X.509");
		return (X509Certificate)factory.generateCertificate(new ByteArrayInputStream(bytes));
	}

	/**
	 * 鍵を Base64 で符号化します。
	 *
	 * @param key 鍵
	 * @return Base64 で符号化された鍵
	 */
	public synchronized String encodeSecretKey(Key key) throws UnsupportedEncodingException
	{
		Base64Encoder encoder = new Base64Encoder();
		return new String(encoder.encode(key.getEncoded()), "ISO-8859-1");
	}

	/**
	 * Base64 で符号化された鍵 (DESede) を復号化します。
	 * 鍵アルゴリズムは DESede 固定です。
	 *
	 * @param base64Key Base64 で符号化された鍵 (DESede)
	 * @return 鍵
	 */
	public synchronized SecretKey decodeSecretKey(String base64Key)
	{
		Base64Decoder decoder = new Base64Decoder();
		byte[] bytes = decoder.decode(base64Key);
		return new SecretKeySpec(bytes, "DESede");
	}

	/**
	 * サブジェクトから CN を取り出します。
	 *
	 * @param subject サブジェクト
	 * @return CN
	 */
	public synchronized String getCommonName(String subject)
	{
		final String prefix = "CN=";
		String commonName = null;
		int ps = subject.indexOf(prefix);
		if (ps >= 0)
		{
			StringBuffer buf = new StringBuffer();
			for (int i = ps + prefix.length(); i < subject.length(); i++)
			{
				if (subject.charAt(i) == ',')
				{
					if (subject.charAt(i - 1) != '\\')
					{
						break;
					}
				}
				buf.append(subject.charAt(i));
			}
			commonName = buf.toString();
		}

		return commonName;
	}

	/**
	 * サブジェクトからカスタマー ID を取り出します。
	 *
	 * @param subject サブジェクト
	 * @return カスタマー ID
	 */
	public synchronized String getCustomerId(String subject)
	{
		final String prefix = "OU=CSTID:";
		String customerId = null;
		int ps = subject.indexOf(prefix);
		if (ps >= 0)
		{
			StringBuffer buf = new StringBuffer();
			for (int i = ps + prefix.length(); i < subject.length(); i++)
			{
				if (subject.charAt(i) == ',')
				{
					if (subject.charAt(i - 1) != '\\')
					{
						break;
					}
				}
				buf.append(subject.charAt(i));
			}
			customerId = buf.toString();
		}

		return customerId;
	}

}
