/*
 * MORPHO GENERATION-F, Open Source Groupware Solutions
 * http://www.mintjams.jp/
 *
 * Copyright (c) 2004, MintJams, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *	  this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *	  this list of conditions and the following disclaimer in the documentation
 *	  and/or other materials provided with the distribution.
 * 3. Neither the name of the MintJams, Inc. nor the names of its contributors
 *	  may be used to endorse or promote products derived from this software
 *	  without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package jp.co.mintjams.generation.base.util;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import jp.co.mintjams.itruth.sql.ResultSetCache;
import jp.co.mintjams.itruth.sql.SQLExecutor;
import jp.co.mintjams.itruth.sql.spec.SQLParameterSpec;
import jp.morpho.util.ConversionList;
import jp.morpho.util.ConversionMap;
import jp.morpho.util.PrimitiveHolderList;
import jp.morpho.util.PrimitiveHolderMap;

/**
 * 検索処理を実行するクラスです。
 * 検索結果は、1 ページ分だけ返されます。
 */
public class Search
{

	/**
	 * 「前方一致」を識別する検索オプションです。
	 */
	public static final int OPTION_STARTS = 1;

	/**
	 * 「部分一致」を識別する検索オプションです。
	 */
	public static final int OPTION_INCLUDE = 2;

	/**
	 * 「後方一致」を識別する検索オプションです。
	 */
	public static final int OPTION_ENDS = 3;

	/**
	 * 「完全一致」を識別する検索オプションです。
	 */
	public static final int OPTION_EQUALS = 4;

	/**
	 * 「不一致」を識別する検索オプションです。
	 */
	public static final int OPTION_NOTEQUALS = 5;

	/**
	 * 「以下」を識別する検索オプションです。
	 */
	public static final int OPTION_UNDER = 6;

	/**
	 * 「以上」を識別する検索オプションです。
	 */
	public static final int OPTION_OVER = 7;

	/**
	 * 基本となる SQL です。
	 */
	private String baseSQL = null;

	/**
	 * 追加の条件式です。
	 */
	private String appendWhere = null;

	/**
	 * 値の型を定義するマップです。
	 */
	private PrimitiveHolderMap columnTypeMap = new ConversionMap();

	/**
	 * 要求パラメータの名前とデータベースの列の名前を関連付けるマップ
	 */
	private PrimitiveHolderMap columnNameMap = new ConversionMap();

	/**
	 * Search を作成します。
	 */
	public Search()
	{
		super();
	}

	/**
	 * テキスト項目の検索オプションリストを返します。
	 *
	 * @return テキスト項目の検索オプションリスト
	 */
	public synchronized List getTextSearchOptionList()
	{
		List items = new Vector();
		PrimitiveHolderMap item = null;

		//前方一致
		item = new ConversionMap();
		item.putString("label", "で始まる");
		item.putInt("data", OPTION_STARTS);
		items.add(item);

		//部分一致
		item = new ConversionMap();
		item.putString("label", "を含む");
		item.putInt("data", OPTION_INCLUDE);
		items.add(item);

		//後方一致
		item = new ConversionMap();
		item.putString("label", "で終わる");
		item.putInt("data", OPTION_ENDS);
		items.add(item);

		//完全一致
		item = new ConversionMap();
		item.putString("label", "と同じ");
		item.putInt("data", OPTION_EQUALS);
		items.add(item);

		//不一致
		item = new ConversionMap();
		item.putString("label", "以外");
		item.putInt("data", OPTION_NOTEQUALS);
		items.add(item);

		return items;
	}

	/**
	 * 数値項目の検索オプションリストを返します。
	 *
	 * @return 数値項目の検索オプションリスト
	 */
	public synchronized List getNumericSearchOptionList()
	{
		List items = new Vector();
		ConversionMap item = null;

		//以下
		item = new ConversionMap();
		item.putString("label", "以下");
		item.putInt("data", OPTION_UNDER);
		items.add(item);

		//完全一致
		item = new ConversionMap();
		item.putString("label", "と等しい");
		item.putInt("data", OPTION_EQUALS);
		items.add(item);

		//以上
		item = new ConversionMap();
		item.putString("label", "以上");
		item.putInt("data", OPTION_OVER);
		items.add(item);

		//不一致
		item = new ConversionMap();
		item.putString("label", "以外");
		item.putInt("data", OPTION_NOTEQUALS);
		items.add(item);

		return items;
	}

	/**
	 * SQL の WHERE 句に使用する式を返します。
	 * PreparedStatement を使用して、パラメータを設定してください。
	 *
	 * @return SQL の WHERE 句に使用する式
	 */
	public synchronized String getExpression(String columnName, int searchOption)
	{
		StringBuffer buf = new StringBuffer();
		buf.append(columnName);

		switch (searchOption)
		{
		/*
		 * 「前方一致」の場合
		 */
		case OPTION_STARTS:
			buf.append(" LIKE ");
			buf.append("?");
			break;

		/*
		 * 「部分一致」の場合
		 */
		case OPTION_INCLUDE:
			buf.append(" LIKE ");
			buf.append("?");
			break;

		/*
		 * 「後方一致」の場合
		 */
		case OPTION_ENDS:
			buf.append(" LIKE ");
			buf.append("?");
			break;

		/*
		 * 「完全一致」の場合
		 */
		case OPTION_EQUALS:
			buf.append(" = ");
			buf.append("?");
			break;

		/*
		 * 「不一致」の場合
		 */
		case OPTION_NOTEQUALS:
			buf.append(" <> ");
			buf.append("?");
			break;

		/*
		 * 「以下」の場合
		 */
		case OPTION_UNDER:
			buf.append(" <= ");
			buf.append("?");
			break;

		/*
		 * 「以上」の場合
		 */
		case OPTION_OVER:
			buf.append(" >= ");
			buf.append("?");
			break;

		/*
		 * その他の場合
		 */
		default:
			buf.append(" = ");
			buf.append("?");
			break;
		}

		return buf.toString();
	}

	/**
	 * SQL の パラメータに使用する値を返します。
	 *
	 * @return SQL の パラメータに使用する値
	 */
	public synchronized String getExpressionValue(String columnValue, int searchOption)
	{
		StringBuffer buf = new StringBuffer();

		switch (searchOption)
		{
		/*
		 * 「前方一致」の場合
		 */
		case OPTION_STARTS:
			buf.append(columnValue).append("%");
			break;

		/*
		 * 「部分一致」の場合
		 */
		case OPTION_INCLUDE:
			buf.append("%").append(columnValue).append("%");
			break;

		/*
		 * 「後方一致」の場合
		 */
		case OPTION_ENDS:
			buf.append("%").append(columnValue);
			break;

		/*
		 * 「完全一致」の場合
		 */
		case OPTION_EQUALS:
			buf.append(columnValue);
			break;

		/*
		 * 「不一致」の場合
		 */
		case OPTION_NOTEQUALS:
			buf.append(columnValue);
			break;

		/*
		 * 「以下」の場合
		 */
		case OPTION_UNDER:
			buf.append(columnValue);
			break;

		/*
		 * 「以上」の場合
		 */
		case OPTION_OVER:
			buf.append(columnValue);
			break;

		/*
		 * その他の場合
		 */
		default:
			buf.append(columnValue);
			break;
		}

		return buf.toString();
	}

	/**
	 * 検索処理を実行します。
	 *
	 * @param form 要求パラメータのマップ
	 * @param conn Connection
	 * @param firstRow キャッシュする先頭行（指定しない場合は 0）
	 * @param lastRow キャッシュする最終行（指定しない場合は 0）
	 * @return ResultSetCache
	 * @throws SQLException データベースアクセスエラーが発生した場合
	 */
	public synchronized ResultSetCache search(PrimitiveHolderMap form, Connection conn, int firstRow, int lastRow) throws SQLException
	{
		/*
		 * SQL を作成します。
		 */
		StringBuffer sql = new StringBuffer();
		sql.append(getBaseSQL());

		/*
		 * 検索条件を設定します。
		 */
		StringBuffer where = new StringBuffer();
		PrimitiveHolderList valueList = new ConversionList();
		PrimitiveHolderList typeList = new ConversionList();
		String[] columnNames = getColumnNames();
		for (int i = 0; i < columnNames.length; i++)
		{
			/*
			 * 画面で入力されるフィールドの名前を取得します。
			 */
			String fieldName = columnNameMap.getString(columnNames[i]);

			/*
			 * 画面で入力されたフィールドの値を取得します。
			 */
			String value = form.getString(fieldName);

			/*
			 * 値が入力されている場合のみ、WHERE 句を作成します。
			 */
			if (value.length() > 0)
			{
				/*
				 * 画面で入力されたフィールドのオプションを取得します。
				 * オプションとは、前方一致、完全一致、後方一致などを表す値です。
				 */
				int optionValue = form.getInt(fieldName + "_option");

				/*
				 * データベースの列の SQL 型を取得します。
				 */
				int sqlType = getColumnType(columnNames[i]);

				/*
				 * 式の先頭に WHERE または AND を付けます。
				 */
				if (where.length() == 0)
				{
					where.append(" WHERE ");
				}
				else
				{
					where.append(" AND ");
				}

				/*
				 * WHERE 句に式を追加します。
				 */
				where.append(getExpression(columnNames[i], optionValue));

				/*
				 * PreparedStatement が設定するパラメータのリストに値を追加します。
				 */
				valueList.addString(getExpressionValue(value, optionValue));

				/*
				 * PreparedStatement が設定するパラメータのリスト SQL 型を追加します。
				 */
				typeList.addInt(sqlType);
			}
		}

		/*
		 * 基本となる SQL に WHERE 句を追加します。
		 */
		sql.append(where.toString());

		/*
		 * 基本となる SQL に 追加の SQL を追加します。
		 */
		if (getAppendWhere() != null)
		{
			if (where.length() == 0)
			{
				sql.append(" WHERE ");
			}
			else
			{
				sql.append(" AND ");
			}
			sql.append(getAppendWhere());
		}

		/*
		 * パラメータを作成します。
		 */
		List paramList = new ArrayList();
		for (int i = 0; i < valueList.size(); i++)
		{
			paramList.add(new SQLParameterSpec(valueList.get(i), typeList.getInt(i)));
		}
		SQLParameterSpec[] paramSpecs = new SQLParameterSpec[paramList.size()];
		paramSpecs = (SQLParameterSpec[])paramList.toArray(paramSpecs);

		try
		{
			/*
			 * SQL を実行します。
			 */
			SQLExecutor executor = new SQLExecutor(conn);
			ResultSetCache rsc = executor.executeQuery(sql.toString(), paramSpecs, firstRow, lastRow);

			/*
			 * ResultSetCache を返します。
			 */
			return rsc;
		}
		catch (SQLException e)
		{
			throw new SQLException(e.toString() + " // " + sql.toString());
		}
	}

	/**
	 * 該当するすべての検索結果を返します。
	 *
	 * @param form 要求パラメータのマップ
	 * @param conn Connection
	 * @return ResultSetCache
	 * @throws SQLException データベースアクセスエラーが発生した場合
	 */
	public synchronized ResultSetCache search(PrimitiveHolderMap form, Connection conn) throws SQLException
	{
		return search(form, conn, 0, 0);
	}

	/**
	 * 基本となる SQL を返します。
	 *
	 * @return 基本となる SQL
	 */
	public String getBaseSQL()
	{
		return baseSQL;
	}

	/**
	 * 基本となる SQL を設定します。
	 * 基本となる SQL とは WHERE 句のない SQL 文です。
	 *
	 * @param sql 基本となる SQL
	 */
	public void setBaseSQL(String sql)
	{
		baseSQL = sql;
	}

	/**
	 * 追加の条件式を返します。
	 *
	 * @return 追加の条件式
	 */
	public String getAppendWhere()
	{
		return appendWhere;
	}

	/**
	 * 追加の条件式を設定します。
	 *
	 * @param sql 追加の条件式
	 */
	public void setAppendWhere(String sql)
	{
		appendWhere = sql;
	}

	/**
	 * WHERE 句で使用するデータベースの列の SQL 型を設定します。
	 *
	 * @param name データベースの列の名前
	 * @param type パラメータの SQL 型
	 */
	public void setColumnType(String name, int type)
	{
		columnTypeMap.putInt(name, type);
	}

	/**
	 * 指定された名前のデータベースの列の SQL 型を返します。
	 *
	 * @param name SQL 型を取得するデータベースの列の名前
	 * @return データベースの列の SQL 型
	 */
	public int getColumnType(String name)
	{
		return columnTypeMap.getInt(name);
	}

	/**
	 * 指定された名前のデータベースの列の SQL 型を削除します。
	 *
	 * @param name SQL 型を削除するデータベースの列の名前
	 */
	public void removeColumnType(String name)
	{
		columnTypeMap.remove(name);
	}

	/**
	 * 指定された要求パラメータの名前と指定されたデータベースの列の名前を関連付けます。
	 *
	 * @param columnName 指定される要求パラメータの名前が関連付けられるデータベースの列の名前
	 * @param paramName 指定されるデータベースの列の名前に関連付けられる要求パラメータの名前
	 */
	public void setColumnName(String columnName, String paramName)
	{
		columnNameMap.putString(columnName, paramName);
	}

	/**
	 * 指定のデータベースの列の名前をマップする要求パラメータの名前を返します。
	 * このデータベースの列の名前のマッピングを保持していない場合は null を返します。
	 *
	 * @param columnName 関連付けられた要求パラメータの名前が返されるデータベースの列の名前
	 * @return 指定されたデータベースの列の名前にマッピングしている要求パラメータの名前。このデータベースの列の名前に対するマッピングがない場合は null
	 */
	public String getColumnName(String columnName)
	{
		return columnNameMap.getString(columnName);
	}

	/**
	 * すべてのデータベースの列の名前が格納されている配列を返します。
	 *
	 * @return すべてのデータベースの列の名前を保持している配列
	 */
	public String[] getColumnNames()
	{
		String[] columnNames = new String[columnNameMap.size()];
		return (String[])columnNameMap.keySet().toArray(columnNames);
	}

	/**
	 * このデータベースの列の名前にマッピングがある場合に、そのマッピングを削除します。
	 *
	 * @param columnName マッピングが削除されるデータベースの列の名前
	 */
	public void removeColumnName(String columnName)
	{
		columnNameMap.remove(columnName);
	}

}
