/*
 * MORPHO GENERATION-F, Open Source Groupware Solutions
 * http://www.mintjams.jp/
 *
 * Copyright (c) 2004, MintJams, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *	  this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *	  this list of conditions and the following disclaimer in the documentation
 *	  and/or other materials provided with the distribution.
 * 3. Neither the name of the MintJams, Inc. nor the names of its contributors
 *	  may be used to endorse or promote products derived from this software
 *	  without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package jp.co.mintjams.generation.base.validator;

import java.util.ArrayList;
import java.util.List;

import jp.co.mintjams.itruth.sql.ResultSetCache;
import jp.co.mintjams.itruth.sql.SQLExecutor;
import jp.co.mintjams.itruth.sql.spec.SQLParameterSpec;
import jp.morpho.util.ConversionList;
import jp.morpho.util.PrimitiveHolderList;
import jp.morpho.webapp.servlet.http.HttpActionRequest;
import jp.morpho.webapp.validator.ValidationException;

/**
 * 同じ値が登録されていないかどうかを、データベースに接続して判定する Validator です。
 * 同じ値が登録されている場合はエラーになります。
 * 但し、比較するフィールドと同じ値の場合はエラーにはなりません。
 */
public class NotExistsValidator extends DatabaseAccessValidator
{

	/**
	 * 判定に使用する SQL 文です。
	 * 1 つの '?' IN パラメータプレースホルダーを含む SQL SELECT 文です。
	 */
	private String sql = null;

	/**
	 * 判定に使用する SQL 文に設定されるパラメータのフィールド名のリストです。
	 */
	private PrimitiveHolderList fieldList = new ConversionList();

	/**
	 * 判定に使用する SQL 文に設定されるパラメータの SQL 型のリストです。
	 */
	private PrimitiveHolderList typeList = new ConversionList();

	/**
	 * 指定された要求オブジェクトの HTTP パラメータの妥当性を検証します。
	 *
	 * @param request 要求オブジェクト
	 * @throws ValidationException 妥当性を検証するときに問題が検出された場合
	 */
	public void validate(HttpActionRequest request) throws ValidationException
	{
		super.validate(request);

		/*
		 * 判定する値をフォームデータから取得します。
		 */
		String value = getFieldValueAsString(request);

		/*
		 * 検証処理を実行するかどうかを判定します。
		 */
		if (!this.isValidate(value))
		{
			return;
		}

		/*
		 * 必須記入の検証を行います。
		 */
		this.validateRequired(value);

		/*
		 * パラメータを作成します。
		 */
		List paramList = new ArrayList();
		for (int i = 0; i < fieldList.size(); i++)
		{
			String paramValue = getFieldValueAsString(request, fieldList.getString(i));
			int paramType = typeList.getInt(i);
			paramList.add(new SQLParameterSpec(paramValue, paramType));
		}
		SQLParameterSpec[] paramSpecs = new SQLParameterSpec[paramList.size()];
		paramSpecs = (SQLParameterSpec[])paramList.toArray(paramSpecs);

		try
		{
			/*
			 * SQL を実行します。
			 */
			SQLExecutor executor = new SQLExecutor(getConnection());
			ResultSetCache rsc = executor.executeQuery(this.sql, paramSpecs, 1, 1);

			/*
			 * 既にデータがある場合はエラーにする
			 */
			if (rsc.toList().size() > 0)
			{
				if (this.getCompareFieldName() == null || this.getCompareFieldName().length() == 0)
				{
					throw new ValidationException("value is duplicate");
				}
				else
				{
					String compareValue = request.getParameter(this.getCompareFieldName());
					if (!value.equals(compareValue))
					{
						throw new ValidationException("value is duplicate");
					}
				}
			}
		}
		catch (ValidationException e)
		{
			throw e;
		}
		catch (Exception e)
		{
			throw new ValidationException(e.getMessage());
		}
	}

	/**
	 * 判定に使用する SQL 文を設定します。
	 *
	 * @param sql 判定に使用する SQL 文
	 */
	public void setSql(String sql)
	{
		this.sql = sql;
	}

	/**
	 * 判定に使用する SQL 文に設定されるパラメータのフィールド名を追加します。
	 *
	 * @param fieldName 判定に使用する SQL 文に設定されるパラメータのフィールド名
	 */
	public void addParameterFieldName(String fieldName)
	{
		this.fieldList.addString(fieldName);
	}

	/**
	 * 判定に使用する SQL 文に設定されるパラメータの SQL 型を追加します。
	 *
	 * @param type 判定に使用する SQL 文に設定されるパラメータの SQL 型
	 */
	public void addParameterType(int type)
	{
		this.typeList.addInt(type);
	}

}
